// EConnect.h
// A module for setting up internet and wireless connections
//
// Copyright (c) 2004-2006 Elements Interactive B.V.
// http://www.edgelib.com
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
// Library definition                                            //
///////////////////////////////////////////////////////////////////

#ifdef  EDGE_INCLUDED      //Can only be included by the main engine
#ifndef ECONNECT_INCLUDED  //Make sure the module isn't included multiple times
#define ECONNECT_INCLUDED

#if defined(DEVICE_WIN32)
	#if defined(DEVICE_DESKTOP) || (DEVICE_WINCE >= 400)
		#include <winsock2.h>
		//#include <ws2bth.h>
	#else
		#include <winsock.h>
	#endif
	#if (DEVICE_WINCE >= 400)
		#if defined(DEVICE_GIZMONDO)
			#include <bt_api.h>
		#endif
		#include <bthapi.h>
		#include <bt_sdp.h>
	#endif
#elif defined(DEVICE_SYMBIAN)
	#include <es_sock.h>
	#include <in_sock.h>
	#if (DEVICE_SYMBIAN >= 610)
		#define DEVICE_SYMBIAN_BLUETOOTH
		#include <bt_sock.h>
		#include <btsdp.h>
		#include <btmanclient.h>
	#endif
#elif defined(DEVICE_LINUX)
	#include <sys/socket.h>
	#include <arpa/inet.h>
	#include <netinet/in.h>
	#include <netdb.h>
#endif

#define ECONN_NONE           0
#define ECONN_SOCKET         1
#define ECONN_BLUETOOTH      2 //Connection types

#define EPING_NONE           0
#define EPING_WAITHOST       1
#define EPING_WAITREPLY      2 //Ping states

#define EHTTP_CONNECTION_NONE       0
#define EHTTP_CONNECTION_KEEPALIVE  1
#define EHTTP_CONNECTION_CLOSE      2 //HTTP connection settings

#define MAX_ECONREADBUF   1024 //Buffer for reading socket data
#define MAX_ECONCLIENTS     16 //Maximum number of clients to accept
#define MAX_MIMETYPES       64 //Maximum number of array elements for mime types

#if !defined(DEVICE_SYMBIAN_BLUETOOTH)
	#define TBTSockAddr TSockAddr
	#define TBTDevAddr int
#endif

///////////////////////////////////////////////////////////////////
// Structures                                                    //
///////////////////////////////////////////////////////////////////

#if defined(DEVICE_SYMBIAN)
	#define EBT_ADDR TBTSockAddr

	typedef struct
	{
		RSocket cls;
		bool active;
	}ESYMBIANSOCKET;
	//A symbian socket incapsulated to contain more information

#elif defined (DEVICE_LINUX)
	#define INVALID_SOCKET    (SOCKET)(~0)
	#define SOCKET_ERROR      (-1)
	#define EBT_ADDR          int
	#define SOCKET            int
	#define MAXGETHOSTSTRUCT  256
	#define LPHOSTENT         struct hostent*

	#define FD_READ           0x01
	#define FD_WRITE          0x02
	#define FD_OOB            0x04
	#define FD_ACCEPT         0x08
	#define FD_CONNECT        0x10
	#define FD_CLOSE          0x20

	typedef struct in_addr IN_ADDR;
	typedef struct in_addr *LPIN_ADDR;
	typedef struct sockaddr SOCKADDR;
	typedef struct sockaddr *LPSOCKADDR;
	typedef struct sockaddr_in SOCKADDR_IN;
	//Structure definition

#else
	#define EBT_ADDR __int64
#endif

typedef struct
{
	char name[64];
	unsigned char port;
	EBT_ADDR address;
}EBLUETOOTH_DEVICEID;
//A bluetooth device ID

typedef struct
{
	char devicename[64];
	char servicename[64];
	unsigned char port;
	EBT_ADDR address;
}EBLUETOOTH_SERVICEID;
//A bluetooth device ID

typedef struct
{
	char contentencoding[16]; //not fully supported
	char contenttype[128];
	char server[256];
	char location[1024];
	char date[48];
	long contentlength;
	unsigned short status;
}EHTTPRESPONSE_INFO;
//An HTTP header

typedef struct
{
	bool acceptencoding_gzip; //not fully supported
	char host[512];
	char path[512];
	char referer[1024];
	char useragent[128];
	char username[128];
	char password[128];
	unsigned char connection;
	int port;
}EHTTPREQUEST_INFO;
//Info structure for an HTTP request

///////////////////////////////////////////////////////////////////
// Symbian interface classes                                     //
///////////////////////////////////////////////////////////////////

//Network capture predefinition
class ClassENetCapture;

#if defined(DEVICE_SYMBIAN)

//Network module class predefinition
class ClassEConnect;

//Bluetooth device discoverer and advertiser predefinitions
#if defined(DEVICE_SYMBIAN_BLUETOOTH)
class CEdgeDeviceDiscoverer;
class CEdgeAdvertiser;
#endif

#endif

///////////////////////////////////////////////////////////////////
// Classes                                                       //
///////////////////////////////////////////////////////////////////

//Forward declaration
class EHTTPRequest;

class ClassEConnect
{
	#ifdef DEVICE_SYMBIAN
		friend class CEdgeDeviceDiscoverer;
	#endif
	friend class EHTTPRequest;
	private:
		#if defined(DEVICE_DESKTOP)
			HANDLE hosthandle;
		#elif defined(DEVICE_WINMOBILE)
			HANDLE cmconn;
		#elif defined(DEVICE_SYMBIAN)
			#if defined(DEVICE_SYMBIAN_BLUETOOTH)
				CEdgeAdvertiser *iAdvertiser;
				CEdgeDeviceDiscoverer *iDiscoverer;
				RBTMan iSecManager;
			#endif
			#if ((DEVICE_SYMBIAN >= 700) && !defined(DEVICE_UIQ)) || (DEVICE_SYMBIAN >= 800)
				RConnection cmconn;
			#endif
			RSocketServ iSocketServer;
			ESYMBIANSOCKET clientsocket[MAX_ECONCLIENTS];
			ESYMBIANSOCKET opensocket;
			TBuf8<MAX_ECONREADBUF> readbuf;
		#endif
		#if !defined(DEVICE_SYMBIAN)
			SOCKET clientsocket[MAX_ECONCLIENTS];
			SOCKET opensocket;
			SOCKADDR_IN sockaddress;
			LPHOSTENT lphostent;
			char hostbuf[MAXGETHOSTSTRUCT];
			char readbuf[MAX_ECONREADBUF];
		#endif
		#if defined(DEVICE_GIZMONDO)
			HMODULE dll_btapi;
		#endif
		ClassEdge *edge;
		ClassENetCapture *capturedapi;
		unsigned char connectid;
		char *httpcommand;
		unsigned long httpcmdsize;
		unsigned long pingtimestamp;
		unsigned short usedprotocol;
		unsigned char pingstate;
		bool isserver;
		bool cmconnopen;
		#if defined(DEVICE_SYMBIAN)
			bool SetChannelSecurity(TBool aAuthentication, TBool aEncryption, TBool aAuthorisation, TInt aChannel);
			bool SetSocketSecurity(TBTSockAddr *addr, TBool aAuthentication, TBool aEncryption, TBool aAuthorisation, TBool aDenied);
		#endif
		void static SplitURL(char *url, char *host, char *page, unsigned long urlend = 0xFFFFFFF);
		static inline bool IsBase64(unsigned char c);
		void ExpandString(char *str, unsigned long &pos, char *str2);
		void GenerateBoundary(char *boundary, unsigned char bsize = 40);
		void EdgeSend(unsigned char id, unsigned char *buffer, long buflen, long &totallen, bool simulate);
		void SendPingData(unsigned char id, bool request);
		void LogStatus(char *msg);
		void CallNetMessage(long id, long msg, long bufsize, unsigned char *buffer);
	public:
		ClassEConnect(void);
		~ClassEConnect(void);
		void SetConnectID(unsigned char id);
		ERESULT Init(unsigned short protocol, ClassEdge *pedge);
		void Cleanup(void);
		void AttachNetCapture(ClassENetCapture *netcapture);
		void DetachNetCapture(void);
		ERESULT EstablishConnection(void);
		void ReleaseConnection(void);
		unsigned long SearchWireless(unsigned long numdevices = 16, bool searchservices = false);
		void StopWirelessSearch(void);
		void OnDeviceFound(EBLUETOOTH_DEVICEID *device, bool endlist);
		void OnServiceFound(EBLUETOOTH_SERVICEID *service);
		bool SearchHost(char *address, int port);
		void CheckSockets(void);
		#if !defined(DEVICE_SYMBIAN)
			void SocketMessage(SOCKET cursocket, bool isclient);
			ERESULT Startup(bool startserver, char *address, int port, ENATIVETYPE winhandle, bool directip = false);
			void OnAsync(unsigned long wparam, int event);
			SOCKET CreateSocket();
			void CloseSocket(SOCKET &cursocket);
		#endif
		void Disconnect(unsigned char connection = 0);
		void Send(unsigned char *buffer, long buflen, unsigned char connection = 0);
		void SendRaw(unsigned char *buffer, long buflen, unsigned char connection = 0);
		void SendHTTPGet(char *url, bool autoencode = false);
		void SendHTTPGet(EHTTPRequest *request);
		void SendHTTPPost(char *url, bool autoencode = false);
		void SendHTTPPost(EHTTPRequest *request);
		void SendHTTPPostMultipart(EHTTPRequest *request);
		void RequestPing(unsigned char connection = 0);
		void RespondPing(unsigned char connection = 0);
		unsigned long GetPingTime(void);
		void static UrlEncode(char *dst, char *src);
		void static UrlDecode(char *dst, char *src);
		void static Base64Encode(char *dst, const char *src);
		void static Base64Decode(char *dst, const char *src);
		void IPAtoA(char *ipstr, unsigned long address);
		unsigned long AtoIPA(char *ipstr);
		#if defined(DEVICE_SYMBIAN)
			void static BTAtoA(char *btstr, EBT_ADDR address);
			EBT_ADDR static AtoBTA(char *btstr);
		#else
			void static BTAtoA(char *btstr, EBT_ADDR address);
			EBT_ADDR static AtoBTA(char *btstr);
		#endif
};//The connection class

class ENetStack
{
	private:
		bool peekcache;
		unsigned char *rcvstack;
		unsigned long stackindex;
		unsigned long totalstacksize;
		unsigned long curstacksize;
		unsigned long lastpeeksize;
		unsigned long lastpeekindex;
		void ScanEdge(unsigned long &pkgstart, unsigned long &pkgsize);
	public:
		ENetStack(void);
		~ENetStack(void);
		bool Alloc(unsigned long stacksize);
		void DeAlloc(void);
		bool Push(void *buffer, unsigned long bufsize);
		unsigned long Peek(void);
		unsigned char Pop(void *buffer, unsigned long &bufsize);
};//The connection data stack class

class EHTTPStack
{
	private:
		EHTTPRESPONSE_INFO httpheader;
		unsigned char *rcvstack;
		unsigned long totalstacksize;
		unsigned long curstacksize;
		unsigned long stackindex;
		unsigned char lrcnt;
		unsigned char lncnt;
		bool compressedbody;
		void ParseHeader(void);
	public:
		EHTTPStack(void);
		~EHTTPStack(void);
		void Clear(void);
		bool Alloc(unsigned long stacksize);
		void DeAlloc(void);
		bool Push(void *buffer, unsigned long bufsize);
		unsigned long BodySize(void);
		bool GetBody(void *buffer, unsigned long &bufsize);
		//bool GetBodyUncompressed(void *buffer, unsigned long &bufsize);
		bool GetResponseInfo(EHTTPRESPONSE_INFO *info);
		bool GetResponseInfo(char *rawinfo);
};//The HTTP header stack class

class EHTTPRequest
{
	friend class ClassEConnect;
	private:
		unsigned long *paramvaluesize;
		unsigned char **paramvalue;
		char **paramfilename;
		char **paramname;
		char **mimetype;
		unsigned long paramcount;
		void DefaultData(void);
		bool ExpandList(unsigned char numblocks);
		void AddMimeArray(unsigned long &index, char extmime[MAX_MIMETYPES][8], char namemime[MAX_MIMETYPES][32], char *ext, char *name);
		void GetMimeType(char *mime, char *filename);
		void GetMimeType(char *mime, WCHAR *filename);
	public:
		EHTTPREQUEST_INFO info;
		EHTTPRequest(void);
		EHTTPRequest(char *seturl);
		~EHTTPRequest(void);
		bool AddParam(char *name, char *value);
		bool AddParam(char *name, long value);
		bool AddData(char *name, unsigned char *buffer, unsigned long bufsize, char *filename = NULL, char *datamimetype = NULL);
		bool AddFile(char *name, char *filename, char *filemimetype = NULL);
		bool AddFile(char *name, ClassEFile *ef, char *filemimetype = NULL);
};//The HTTP request class

#endif
#endif
