// EMath.h
// A math module for fixed point math
//
// Copyright (c) 2004-2007 Elements Interactive B.V.
// http://www.edgelib.com
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
// Library definition                                            //
///////////////////////////////////////////////////////////////////

#ifdef  EDGE_INCLUDED   //Can only be included by the main engine
#ifndef EMATH_INCLUDED  //Make sure the module isn't included multiple times
#define EMATH_INCLUDED

#define EMATH_FIXEDPI   205887

///////////////////////////////////////////////////////////////////
// Classes                                                       //
///////////////////////////////////////////////////////////////////

class ClassEMath
{
	private:
		long *etblsin;
		long *etblcos;
		unsigned long randseed;
	public:
		ClassEMath(void);
		~ClassEMath(void);
		void Init(void);
		long Sin(unsigned long index);
		long Cos(unsigned long index);
		long Tan(unsigned long index);
		long Cotan(unsigned long index);
		void SetRandSeed(unsigned long seed);
		unsigned long GetRandSeed(void);
		unsigned long Rand();
		unsigned long Rand(long min, long max);
		long static Abs(long value);
		unsigned long static IntSqrt(long value);
		long static FloatToFixed(unsigned long value, int fixedprecision = 16);
		unsigned long static FixedToFloat(long value, int fixedprecision = 16);
		void static MatrixLoadIdentity(long *matrix);
		void MatrixSetRotate(unsigned short x, unsigned short y, unsigned short z, long *matrix);
		void static MatrixSetTranslate(long x, long y, long z, long *matrix);
		void static MatrixMultiply(long *dst, long *src);
		void static MatrixReverse(long *matrix);
};

#endif
#endif
